r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Events
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class SchemaVersionInstance(InstanceResource):
    def __init__(self, version, payload, id: str, schema_version: Optional[int] = None):
        """
        Initialize the SchemaVersionInstance

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        super().__init__(version)

        self._properties = {
            "id": payload.get("id"),
            "schema_version": deserialize.integer(payload.get("schema_version")),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "url": payload.get("url"),
            "raw": payload.get("raw"),
        }

        self._solution = {
            "id": id,
            "schema_version": schema_version or self._properties["schema_version"],
        }
        self._context: Optional[SchemaVersionContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: SchemaVersionContext for this SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        if self._context is None:
            self._context = SchemaVersionContext(
                self._version,
                id=self._solution["id"],
                schema_version=self._solution["schema_version"],
            )
        return self._context

    @property
    def id(self):
        """
        :returns: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.
        :rtype: str
        """
        return self._properties["id"]

    @property
    def schema_version(self):
        """
        :returns: The version of this schema.
        :rtype: int
        """
        return self._properties["schema_version"]

    @property
    def date_created(self):
        """
        :returns: The date the schema version was created, given in ISO 8601 format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def raw(self):
        """
        :returns:
        :rtype: str
        """
        return self._properties["raw"]

    def fetch(self):
        """
        Fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Events.V1.SchemaVersionInstance {}>".format(context)


class SchemaVersionContext(InstanceContext):
    def __init__(self, version: Version, id: str, schema_version: int):
        """
        Initialize the SchemaVersionContext

        :param Version version: Version that contains the resource
        :param id: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.
        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "id": id,
            "schema_version": schema_version,
        }
        self._uri = "/Schemas/{id}/Versions/{schema_version}".format(**self._solution)

    def fetch(self):
        """
        Fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return SchemaVersionInstance(
            self._version,
            payload,
            id=self._solution["id"],
            schema_version=self._solution["schema_version"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return SchemaVersionInstance(
            self._version,
            payload,
            id=self._solution["id"],
            schema_version=self._solution["schema_version"],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Events.V1.SchemaVersionContext {}>".format(context)


class SchemaVersionPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of SchemaVersionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return SchemaVersionInstance(self._version, payload, id=self._solution["id"])

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Events.V1.SchemaVersionPage>"


class SchemaVersionList(ListResource):
    def __init__(self, version: Version, id: str):
        """
        Initialize the SchemaVersionList

        :param Version version: Version that contains the resource
        :param id: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionList
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "id": id,
        }
        self._uri = "/Schemas/{id}/Versions".format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams SchemaVersionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams SchemaVersionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists SchemaVersionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists SchemaVersionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return SchemaVersionPage(self._version, response, self._solution)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return SchemaVersionPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return SchemaVersionPage(self._version, response, self._solution)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return SchemaVersionPage(self._version, response, self._solution)

    def get(self, schema_version):
        """
        Constructs a SchemaVersionContext

        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        return SchemaVersionContext(
            self._version, id=self._solution["id"], schema_version=schema_version
        )

    def __call__(self, schema_version):
        """
        Constructs a SchemaVersionContext

        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        return SchemaVersionContext(
            self._version, id=self._solution["id"], schema_version=schema_version
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Events.V1.SchemaVersionList>"
